@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2017	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -38,15 +38,13 @@
 #include "utils/elog.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
-#include "utils/pool_path.h"
 
 #include "version.h"
 #include "auth/pool_passwd.h"
 #include "query_cache/pool_memqcache.h"
-#include "watchdog/wd_ext.h"
+#include "watchdog/wd_utils.h"
 
 static void daemonize(void);
-static char	*get_pid_file_path(void);
 static int read_pid_file(void);
 static void write_pid_file(void);
 static void usage(void);
@@ -54,10 +52,9 @@ static void show_version(void);
 static void stop_me(void);
 static void FileUnlink(int code, Datum path);
 
-char *pcp_conf_file = NULL; /* absolute path of the pcp.conf */
-char *conf_file = NULL;		/* absolute path of the pgpool.conf */
-char *hba_file = NULL;		/* absolute path of the hba.conf */
-char *base_dir = NULL;		/* The working dir from where pgpool was invoked from */
+char pcp_conf_file[POOLMAXPATHLEN+1]; /* path for pcp.conf */
+char conf_file[POOLMAXPATHLEN+1];
+char hba_file[POOLMAXPATHLEN+1];
 
 static int not_detach = 0;		/* non 0 if non detach option (-n) is given */
 int stop_sig = SIGTERM;		/* stopping signal default value */
@@ -72,12 +69,9 @@ int main(int argc, char **argv)
 	bool discard_status = false;
 	bool clear_memcache_oidmaps = false;
 
-	char pcp_conf_file_path[POOLMAXPATHLEN+1];
-	char conf_file_path[POOLMAXPATHLEN+1];
-	char hba_file_path[POOLMAXPATHLEN+1];
-
 	static struct option long_options[] = {
 		{"hba-file", required_argument, NULL, 'a'},
+		{"clear", no_argument, NULL, 'c'},
 		{"debug", no_argument, NULL, 'd'},
 		{"config-file", required_argument, NULL, 'f'},
 		{"pcp-file", required_argument, NULL, 'F'},
@@ -94,9 +88,9 @@ int main(int argc, char **argv)
 	myargc = argc;
 	myargv = argv;
 
-	snprintf(conf_file_path, sizeof(conf_file_path), "%s/%s", DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
-	snprintf(pcp_conf_file_path, sizeof(pcp_conf_file_path), "%s/%s", DEFAULT_CONFIGDIR, PCP_PASSWD_FILE_NAME);
-	snprintf(hba_file_path, sizeof(hba_file_path), "%s/%s", DEFAULT_CONFIGDIR, HBA_CONF_FILE_NAME);
+	snprintf(conf_file, sizeof(conf_file), "%s/%s", DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
+	snprintf(pcp_conf_file, sizeof(pcp_conf_file), "%s/%s", DEFAULT_CONFIGDIR, PCP_PASSWD_FILE_NAME);
+	snprintf(hba_file, sizeof(hba_file), "%s/%s", DEFAULT_CONFIGDIR, HBA_CONF_FILE_NAME);
     while ((opt = getopt_long(argc, argv, "a:df:F:hm:nDCxv", long_options, &optindex)) != -1)
 	{
 		switch (opt)
@@ -107,7 +101,7 @@ int main(int argc, char **argv)
 					usage();
 					exit(1);
 				}
-				strlcpy(hba_file_path, optarg, sizeof(hba_file_path));
+				strlcpy(hba_file, optarg, sizeof(hba_file));
 				break;
 
 			case 'x':	/* enable cassert */
@@ -124,7 +118,7 @@ int main(int argc, char **argv)
 					usage();
 					exit(1);
 				}
-				strlcpy(conf_file_path, optarg, sizeof(conf_file_path));
+				strlcpy(conf_file, optarg, sizeof(conf_file));
 				break;
 
 			case 'F':   /* specify PCP password file */
@@ -133,7 +127,7 @@ int main(int argc, char **argv)
 					usage();
 					exit(1);
 				}
-				strlcpy(pcp_conf_file_path, optarg, sizeof(pcp_conf_file_path));
+				strlcpy(pcp_conf_file, optarg, sizeof(pcp_conf_file));
 				break;
 
 			case 'h':
@@ -183,11 +177,7 @@ int main(int argc, char **argv)
 	}
 #ifdef USE_SSL
 	/* global ssl init */
-#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
-	OPENSSL_init_ssl(0, NULL);
-#else
 	SSL_library_init();
-#endif
 	SSL_load_error_strings();
 #endif /* USE_SSL */
 
@@ -195,13 +185,6 @@ int main(int argc, char **argv)
 	/* create MemoryContexts */
 	MemoryContextInit();
 
-	/* load the CWD before it is changed */
-	base_dir = get_current_working_dir();
-	/* convert all the paths to absolute paths*/
-	conf_file = make_absolute_path(conf_file_path, base_dir);
-	pcp_conf_file = make_absolute_path(pcp_conf_file_path, base_dir);
-	hba_file = make_absolute_path(hba_file_path, base_dir);
-
 	mypid = getpid();
 
 	pool_init_config();
@@ -221,6 +204,9 @@ int main(int argc, char **argv)
     if(debug_level > 0 && pool_config->log_min_messages > DEBUG1)
         pool_config->log_min_messages = DEBUG1;
 
+	if (pool_config->enable_pool_hba)
+		load_hba(hba_file);
+
 	/*
 	 * If a non-switch argument remains, then it should be either "reload" or "stop".
 	 */
@@ -249,6 +235,7 @@ int main(int argc, char **argv)
 		if (!strcmp(argv[optind], "stop"))
 		{
 			stop_me();
+			unlink(pool_config->pid_file_name);
 			exit(0);
 		}
 		else
@@ -284,9 +271,6 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	if (pool_config->enable_pool_hba)
-		load_hba(hba_file);
-
 	/* check effective user id for watchdog */
 	/* watchdog must be started under the privileged user */
 	wd_check_network_command_configurations();
@@ -299,7 +283,7 @@ int main(int argc, char **argv)
 	else
 		daemonize();
 	
-#ifdef HAVE_SYSLOG
+#ifdef HAVE_VSYSLOG
     set_syslog_parameters(pool_config->syslog_ident ? pool_config->syslog_ident : "pgpool",
                           pool_config->syslog_facility);
 #endif
@@ -318,7 +302,7 @@ int main(int argc, char **argv)
 		dirp = dirname(dirnamebuf);
 		snprintf(pool_passwd, sizeof(pool_passwd), "%s/%s",
 				 dirp, pool_config->pool_passwd);
-		pool_init_pool_passwd(pool_passwd);
+		pool_init_pool_passwd(pool_passwd, POOL_PASSWD_R);
 	}
 
 	pool_semaphore_create(MAX_NUM_SEMAPHORES);
@@ -346,17 +330,17 @@ static void usage(void)
 	fprintf(stderr, "  pgpool [ -f CONFIG_FILE ] [ -F PCP_CONFIG_FILE ] [ -a HBA_CONFIG_FILE ] reload\n\n");
 	fprintf(stderr, "Common options:\n");
 	fprintf(stderr, "  -a, --hba-file=HBA_CONFIG_FILE\n");
-	fprintf(stderr, "                      Set the path to the pool_hba.conf configuration file\n");
+	fprintf(stderr, "                      Sets the path to the pool_hba.conf configuration file\n");
 	fprintf(stderr, "                      (default: %s/%s)\n",DEFAULT_CONFIGDIR, HBA_CONF_FILE_NAME);
 	fprintf(stderr, "  -f, --config-file=CONFIG_FILE\n");
-	fprintf(stderr, "                      Set the path to the pgpool.conf configuration file\n");
+	fprintf(stderr, "                      Sets the path to the pgpool.conf configuration file\n");
 	fprintf(stderr, "                      (default: %s/%s)\n",DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
 	fprintf(stderr, "  -F, --pcp-file=PCP_CONFIG_FILE\n");
-	fprintf(stderr, "                      Set the path to the pcp.conf configuration file\n");
+	fprintf(stderr, "                      Sets the path to the pcp.conf configuration file\n");
 	fprintf(stderr, "                      (default: %s/%s)\n",DEFAULT_CONFIGDIR, PCP_PASSWD_FILE_NAME);
-	fprintf(stderr, "  -h, --help          Print this help\n\n");
+	fprintf(stderr, "  -h, --help          Prints this help\n\n");
 	fprintf(stderr, "Start options:\n");
-	fprintf(stderr, "  -C, --clear-oidmaps Clear query cache oidmaps when memqcache_method is memcached\n");
+	fprintf(stderr, "  -C, --clear-oidmaps Clears query cache oidmaps when memqcache_method is memcached\n");
 	fprintf(stderr, "                      (If shmem, discards whenever pgpool starts.)\n");
 	fprintf(stderr, "  -n, --dont-detach   Don't run in daemon mode, does not detach control tty\n");
 	fprintf(stderr, "  -x, --debug-assertions   Turns on various assertion checks, This is a debugging aid\n");
@@ -439,7 +423,6 @@ static void daemonize(void)
 static void stop_me(void)
 {
 	pid_t pid;
-	char *pid_file;
 
 	pid = read_pid_file();
 	if (pid < 0)
@@ -463,49 +446,6 @@ static void stop_me(void)
 		sleep(1);
 	}
 	fprintf(stderr, "done.\n");
-	pid_file = get_pid_file_path();
-	unlink(pid_file);
-	pfree(pid_file);
-}
-
-/*
- * The function returns the palloc'd copy of pid_file_path,
- * caller must free it after use
- */
-static char *get_pid_file_path(void)
-{
-	char *new = NULL;
-	if (!is_absolute_path(pool_config->pid_file_name))
-	{
-		/*
-		 * some implementations of dirname() may modify the
-		 * string argument passed to it, so do not use the original
-		 * conf_file as an argument
-		 */
-		char *conf_file_copy = pstrdup(conf_file);
-		char *conf_dir = dirname(conf_file_copy);
-		size_t  path_size;
-		if (conf_dir == NULL)
-		{
-			ereport(LOG,
-					(errmsg("failed to get the dirname of pid file:\"%s\". reason: %s",
-				   pool_config->pid_file_name, strerror(errno))));
-			return NULL;
-		}
-		path_size = strlen(conf_dir) + strlen(pool_config->pid_file_name) + 1 + 1;
-		new = palloc(path_size);
-		snprintf(new, path_size, "%s/%s",conf_dir, pool_config->pid_file_name);
-
-		ereport(DEBUG1,
-				(errmsg("pid file location is \"%s\"",
-						new)));
-	}
-	else
-	{
-		new = pstrdup(pool_config->pid_file_name);
-	}
-
-	return new;
 }
 
 /*
@@ -516,25 +456,15 @@ static int read_pid_file(void)
 	int fd;
 	int readlen;
 	char pidbuf[128];
-	char *pid_file = get_pid_file_path();
 
-	if (pid_file == NULL)
-	{
-		ereport(FATAL,
-				(errmsg("failed to read pid file"),
-				 errdetail("failed to get pid file path from \"%s\"",
-						   pool_config->pid_file_name)));
-	}
-	fd = open(pid_file, O_RDONLY);
+	fd = open(pool_config->pid_file_name, O_RDONLY);
 	if (fd == -1)
 	{
-		pfree(pid_file);
 		return -1;
 	}
 	if ((readlen = read(fd, pidbuf, sizeof(pidbuf))) == -1)
 	{
 		close(fd);
-		pfree(pid_file);
 		ereport(FATAL,
 			(errmsg("could not read pid file as \"%s\". reason: %s",
 				   pool_config->pid_file_name, strerror(errno))));
@@ -542,12 +472,10 @@ static int read_pid_file(void)
 	else if (readlen == 0)
 	{
 		close(fd);
-		pfree(pid_file);
 		ereport(FATAL,
 			(errmsg("EOF detected while reading pid file \"%s\". reason: %s",
 				   pool_config->pid_file_name, strerror(errno))));
 	}
-	pfree(pid_file);
 	close(fd);
 	return(atoi(pidbuf));
 }
@@ -559,17 +487,8 @@ static void write_pid_file(void)
 {
 	int fd;
 	char pidbuf[128];
-	char *pid_file = get_pid_file_path();
 
-	if (pid_file == NULL)
-	{
-		ereport(FATAL,
-				(errmsg("failed to write pid file"),
-				 errdetail("failed to get pid file path from \"%s\"",
-						pool_config->pid_file_name)));
-	}
-
-	fd = open(pid_file, O_CREAT|O_WRONLY, S_IRUSR|S_IWUSR);
+	fd = open(pool_config->pid_file_name, O_CREAT|O_WRONLY, S_IRUSR|S_IWUSR);
 	if (fd == -1)
 	{
 		ereport(FATAL,
@@ -580,7 +499,6 @@ static void write_pid_file(void)
 	if (write(fd, pidbuf, strlen(pidbuf)+1) == -1)
 	{
 		close(fd);
-		pfree(pid_file);
 		ereport(FATAL,
 			(errmsg("could not write pid file as %s. reason: %s",
 				   pool_config->pid_file_name, strerror(errno))));
@@ -588,20 +506,18 @@ static void write_pid_file(void)
 	if (fsync(fd) == -1)
 	{
 		close(fd);
-		pfree(pid_file);
 		ereport(FATAL,
 			(errmsg("could not fsync pid file as %s. reason: %s",
 				   pool_config->pid_file_name, strerror(errno))));
 	}
 	if (close(fd) == -1)
 	{
-		pfree(pid_file);
 		ereport(FATAL,
 			(errmsg("could not close pid file as %s. reason: %s",
 				   pool_config->pid_file_name, strerror(errno))));
 	}
 	/* register the call back to delete the pid file at system exit */
-	on_proc_exit(FileUnlink, (Datum) pid_file);
+	on_proc_exit(FileUnlink, (Datum) pool_config->pid_file_name);
 }
 
 /*
