@@ -131,6 +131,7 @@ static int find_primary_node_repeatedly(void);
 static void terminate_all_childrens();
 static void system_will_go_down(int code, Datum arg);
 static char* process_name_from_pid(pid_t pid);
+static void initialize_backend_status_from_watchdog(void);
 
 static struct sockaddr_un un_addr;		/* unix domain socket path */
 static struct sockaddr_un pcp_un_addr;  /* unix domain socket path for PCP */
@@ -222,13 +223,12 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 	pool_signal(SIGPIPE, SIG_IGN);
 
 	/* create unix domain socket */
-	fds = malloc(sizeof(int));
+	fds = malloc(sizeof(int) * 2);
 	if (fds == NULL)
 		ereport(FATAL,
 				(errmsg("failed to allocate memory in startup process")));
 
 	initialize_shared_mem_objects(clear_memcache_oidmaps);
-
 	if (pool_config->use_watchdog)
 	{
 		sigset_t mask;
@@ -239,14 +239,26 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 		 * when wathcodg fails to start with FATAL, the process
 		 * exits and SIGCHLD is fired, so SIGCHLD handelr is also
 		 * needed.
+		 * Finally, we also need to set the SIGUSR1 handler for the
+		 * failover requests from other watchdog nodes.
+		 * In case a request arrives at the same time when the
+		 * watchdog has just been initialized.
 		 */
 		pool_signal(SIGUSR2, wakeup_handler);
 		pool_signal(SIGCHLD, reap_handler);
+		pool_signal(SIGUSR1, failover_handler);
+
 		/*
 		 * okay as we need to wait until watchdog is in stable state
 		 * so only wait for SIGUSR2, SIGCHLD, and signals those are
 		 * necessary to make sure we respond to user requests of shutdown
-		 if it arives while we are in waiting state.
+		 * if it arrives while we are in waiting state.
+		 *
+		 * Note that SIGUSR1 does not need to be in the wait signal list,
+		 * although it's signal handler is already installed, but even if
+		 * the SIGUSR1 arrives while watchdog is initializing we will continue
+		 * with our normal initialization and will process the failover request
+		 * once our backend status will be synchronized across the cluster
 		 */
 		sigfillset(&mask);
 		sigdelset(&mask, SIGUSR2);
@@ -275,9 +287,13 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 		 * initialize the lifecheck process
 		 */
 		wd_lifecheck_pid = initialize_watchdog_lifecheck();
+
+		/* load the backend node status from watchdog cluster */
+		initialize_backend_status_from_watchdog();
 	}
 
 	fds[0] = create_unix_domain_socket(un_addr);
+	fds[1] = -1;
 	on_proc_exit(FileUnlink, (Datum) un_addr.sun_path);
 
 	/* create inet domain socket if any */
@@ -352,8 +368,10 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 	/* Add onproc exit to clean up the unix domain socket at exit */
 	on_proc_exit(FileUnlink, (Datum)pcp_un_addr.sun_path);
 
-    /* maybe change "*" to pool_config->pcp_listen_addresses */
-	pcp_inet_fd = create_inet_domain_socket("*", pool_config->pcp_port);
+	if (pool_config->pcp_listen_addresses[0])
+	{
+		pcp_inet_fd = create_inet_domain_socket(pool_config->pcp_listen_addresses, pool_config->pcp_port);
+	}
 	pcp_pid = pcp_fork_a_child(pcp_unix_fd, pcp_inet_fd, pcp_conf_file);
 
 	/* Fork worker process */
@@ -369,8 +387,14 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 	ereport(LOG,
 			(errmsg("%s successfully started. version %s (%s)", PACKAGE, VERSION, PGPOOLVERSION)));
 
-	/* Save primary node id */
-	Req_info->primary_node_id = find_primary_node();
+	/*
+	 * if the primary node id is not loaded by watchdog, search for it
+	 */
+	if (Req_info->primary_node_id < 0)
+	{
+		/* Save primary node id */
+		Req_info->primary_node_id = find_primary_node();
+	}
 
 	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
@@ -912,19 +936,24 @@ static int create_inet_domain_socket(const char *hostname, const int port)
 
 	addr.sin_port = htons(port);
 	len = sizeof(struct sockaddr_in);
+
 	status = bind(fd, (struct sockaddr *)&addr, len);
 	if (status == -1)
 	{
-		char *host = "", *serv = "";
+		int saved_errno = errno;
 		char hostname[NI_MAXHOST], servname[NI_MAXSERV];
-		if (getnameinfo((struct sockaddr *) &addr, len, hostname, sizeof(hostname), servname, sizeof(servname), 0) == 0)
+		if ((status = getnameinfo((struct sockaddr *) &addr, len, hostname, sizeof(hostname), servname, sizeof(servname), 0)))
 		{
-			host = hostname;
-			serv = servname;
+			ereport(NOTICE,
+				(errmsg("getnameinfo failed while creating INET domain socket"),
+					 errdetail("getnameinfo failed with reason: \"%s\"",gai_strerror(status))));
+
+			snprintf(servname, sizeof(servname), "%d",port);
+			snprintf(hostname, sizeof(hostname), "%s",hostname);
 		}
 		ereport(FATAL,
 			(errmsg("failed to create INET domain socket"),
-             errdetail("bind on host:\"%s\" server:\"%s\" failed with error \"%s\"",host, serv,strerror(errno))));
+             errdetail("bind on host:\"%s\" server:\"%s\" failed with error \"%s\"",hostname, servname,strerror(saved_errno))));
 	}
 
     backlog = pool_config->num_init_children * pool_config->listen_backlog_multiplier;
@@ -1024,8 +1053,10 @@ static void terminate_all_childrens()
     /* wait for all children to exit */
     do
     {
-        wpid = wait(NULL);
-    }while (wpid > 0 || (wpid == -1 && errno == EINTR));
+		int ret_pid;
+
+        wpid = waitpid(-1, &ret_pid, WNOHANG);
+    } while (wpid > 0 || (wpid == -1 && errno == EINTR));
 
     if (wpid == -1 && errno != ECHILD)
         ereport(LOG,
@@ -1289,6 +1320,8 @@ static RETSIGTYPE exit_handler(int sig)
 {
 	int i;
     pid_t wpid;
+
+	int save_errno = errno;
 	POOL_SETMASK(&AuthBlockSig);
 
 	/*
@@ -1320,6 +1353,7 @@ static RETSIGTYPE exit_handler(int sig)
 				 errdetail("ignoring...")));
 
 		POOL_SETMASK(&UnBlockSig);
+		errno = save_errno;
 		return;
 	}
 	exiting = 1;
@@ -1365,8 +1399,10 @@ static RETSIGTYPE exit_handler(int sig)
 	POOL_SETMASK(&UnBlockSig);
     do
     {
-        wpid = wait(NULL);
-    }while (wpid > 0 || (wpid == -1 && errno == EINTR));
+		int ret_pid;
+
+        wpid = waitpid(-1, &ret_pid, WNOHANG);
+    } while (wpid > 0 || (wpid == -1 && errno == EINTR));
 
     if (wpid == -1 && errno != ECHILD)
         ereport(LOG,
@@ -1374,6 +1410,8 @@ static RETSIGTYPE exit_handler(int sig)
 
 	process_info = NULL;
 	exit(0);
+
+	errno = save_errno;
 }
 
 /*
@@ -1416,6 +1454,8 @@ static int get_next_master_node(void)
  */
 static RETSIGTYPE failover_handler(int sig)
 {
+	int save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	failover_request = 1;
 	if(write(pipe_fds[1], "\0", 1) < 0)
@@ -1423,6 +1463,8 @@ static RETSIGTYPE failover_handler(int sig)
                 (errmsg("failover_handler: write to pipe failed with error \"%s\"", strerror(errno))));
 
 	POOL_SETMASK(&UnBlockSig);
+
+	errno = save_errno;
 }
 
 
@@ -1693,7 +1735,7 @@ static void failover(void)
 			reqkind == NODE_UP_REQUEST)
 		{
 			ereport(LOG,
-					(errmsg("Do not restart children because we are failbacking node id %d host%s port:%d and we are in streaming replication mode", node_id,
+					(errmsg("Do not restart children because we are failbacking node id %d host: %s port: %d and we are in streaming replication mode", node_id,
 					 BACKEND_INFO(node_id).backend_hostname,
 					 BACKEND_INFO(node_id).backend_port)));
 
@@ -1978,9 +2020,11 @@ static void failover(void)
  */
 static RETSIGTYPE health_check_timer_handler(int sig)
 {
+	int save_errno = errno;
 	POOL_SETMASK(&BlockSig);
 	health_check_timer_expired = 1;
 	POOL_SETMASK(&UnBlockSig);
+	errno = save_errno;
 }
 
 
@@ -2069,6 +2113,8 @@ do_health_check(bool use_template_db, volatile int *health_check_node_id)
  */
 static RETSIGTYPE reap_handler(int sig)
 {
+	int save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	sigchld_request = 1;
 	if(pipe_fds[1] && write(pipe_fds[1], "\0", 1) < 0)
@@ -2076,6 +2122,8 @@ static RETSIGTYPE reap_handler(int sig)
             (errmsg("reap_handler: write to pipe failed with error \"%s\"", strerror(errno))));
 
 	POOL_SETMASK(&UnBlockSig);
+
+	errno = save_errno;
 }
 
 /*
@@ -2124,7 +2172,7 @@ static char* process_name_from_pid(pid_t pid)
  * terminating child but shutdowns the pgpool-II. This allow
  * the child process to inform parent process of fatal failures which needs
  * to be rectified (e.g startup failure) by user for smooth running of system.
- * Also the child exits with success status POOL_EXIT_SUCCESS does not gets
+ * Also the child exits with success status POOL_EXIT_NO_RESTART does not gets
  * restarted.
  */
 static void reaper(void)
@@ -2158,6 +2206,7 @@ static void reaper(void)
 		pid_t new_pid = 0;
 		bool shutdown_system = false;
 		bool restart_child = true;
+		bool found = false;
 		char *exiting_process_name = process_name_from_pid(pid);
 
 		/*
@@ -2171,7 +2220,7 @@ static void reaper(void)
 						(errmsg("%s process with pid: %d exit with FATAL ERROR. pgpool-II will be shutdown",exiting_process_name, pid)));
 				shutdown_system = true;
 			}
-			else if(WEXITSTATUS(status) == POOL_EXIT_SUCCESS)
+			else if(WEXITSTATUS(status) == POOL_EXIT_NO_RESTART)
 			{
 				ereport(DEBUG1,
 						(errmsg("%s process with pid: %d exit with SUCCESS. child will not be restarted",exiting_process_name, pid)));
@@ -2196,6 +2245,7 @@ static void reaper(void)
 		/* if exiting child process was PCP handler */
 		if (pid == pcp_pid)
 		{
+			found = true;
 			if(restart_child)
 			{
 				pcp_pid = pcp_fork_a_child(pcp_unix_fd, pcp_inet_fd, pcp_conf_file);
@@ -2208,6 +2258,7 @@ static void reaper(void)
 		/* exiting process was worker process */
 		else if (pid == worker_pid)
 		{
+			found = true;
 			if(restart_child)
 			{
 				worker_pid = worker_fork_a_child();
@@ -2222,6 +2273,7 @@ static void reaper(void)
 		{
 			if (watchdog_pid == pid)
 			{
+				found = true;
 				if(restart_child)
 				{
 					watchdog_pid = initialize_watchdog();
@@ -2232,6 +2284,7 @@ static void reaper(void)
 			}
 			else if (wd_lifecheck_pid == pid)
 			{
+				found = true;
 				if(restart_child)
 				{
 					wd_lifecheck_pid = initialize_watchdog_lifecheck();
@@ -2241,13 +2294,17 @@ static void reaper(void)
 					wd_lifecheck_pid = 0;
 			}
 		}
-		else
+		/* we are not able to identify the exiting process yet.
+		 * check if the exiting process was child process (handling PG clients)
+		 */
+		if (found == false)
 		{
 			/* look for exiting child's pid */
 			for (i=0;i<pool_config->num_init_children;i++)
 			{
 				if (pid == process_info[i].pid)
 				{
+					found = true;
 					/* if found, fork a new child */
 					if (!switching && !exiting && restart_child)
 					{
@@ -2264,6 +2321,7 @@ static void reaper(void)
 		if(shutdown_system)
 			ereport(FATAL,
 				(errmsg("%s process exit with fatal error. exiting pgpool-II",exiting_process_name)));
+
 		else if(restart_child && new_pid)
 		{
 			/* Report if the child was restarted */
@@ -2347,6 +2405,8 @@ static void wakeup_children(void)
 
 static RETSIGTYPE wakeup_handler(int sig)
 {
+	int save_errno = errno;
+
 	wakeup_request = 1;
 	if (processState != INITIALIZING)
 	{
@@ -2356,6 +2416,7 @@ static RETSIGTYPE wakeup_handler(int sig)
 				(errmsg("wakeup_handler: write to pipe failed with error \"%s\"", strerror(errno))));
 		POOL_SETMASK(&UnBlockSig);
 	}
+	errno = save_errno;
 }
 
 /*
@@ -2364,6 +2425,8 @@ static RETSIGTYPE wakeup_handler(int sig)
  */
 static RETSIGTYPE reload_config_handler(int sig)
 {
+	int save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	reload_config_request = 1;
 	if(write(pipe_fds[1], "\0", 1) < 0)
@@ -2371,6 +2434,8 @@ static RETSIGTYPE reload_config_handler(int sig)
             (errmsg("reload_config_handler: write to pipe failed with error \"%s\"", strerror(errno))));
 
 	POOL_SETMASK(&UnBlockSig);
+
+	errno = save_errno;
 }
 
 static void kill_all_children(int sig)
@@ -2825,7 +2890,6 @@ static void initialize_shared_mem_objects(bool clear_memcache_oidmaps)
 	ereport(DEBUG1,
 			(errmsg("Request info are: sizeof(POOL_REQUEST_INFO) %zu bytes requested for shared memory",
 					sizeof(POOL_REQUEST_INFO))));
-
 	/*
 	 * Initialize backend status area.
 	 * From now on, VALID_BACKEND macro can be used.
@@ -2842,6 +2906,7 @@ static void initialize_shared_mem_objects(bool clear_memcache_oidmaps)
 	Req_info->conn_counter = 0;
 	Req_info->switching = false;
 	Req_info->request_queue_head = Req_info->request_queue_tail = -1;
+	Req_info->primary_node_id = -2;
 	InRecovery = pool_shared_memory_create(sizeof(int));
 	*InRecovery = RECOVERY_INIT;
 
@@ -3206,3 +3271,73 @@ int pool_frontend_exists(void)
 		return pg_frontend_exists();
 	return -1;
 }
+
+static void initialize_backend_status_from_watchdog(void)
+{
+	if (pool_config->use_watchdog)
+	{
+		WDPGBackendStatus* backendStatus = get_pg_backend_status_from_master_wd_node();
+		if (backendStatus)
+		{
+			if (backendStatus->node_count <= 0)
+			{
+				/*
+				 * -ve node count is returned by watchdog when the node itself is a master
+				 * and in that case we need to use the loacl backend node status
+				 */
+				ereport(LOG,
+						(errmsg("I am the master watchdog node"),
+						 errdetail("using the local backend node status")));
+			}
+			else
+			{
+				int i;
+				bool reload_maste_node_id = false;
+				ereport(LOG,
+						(errmsg("master watchdog node \"%s\" returned status for %d backend nodes",backendStatus->nodeName,backendStatus->node_count)));
+				ereport(LOG,
+						(errmsg("primary node on master watchdog node \"%s\" is %d",backendStatus->nodeName,backendStatus->primary_node_id)));
+
+				Req_info->primary_node_id = backendStatus->primary_node_id;
+
+				for (i = 0; i < backendStatus->node_count; i++)
+				{
+					if (backendStatus->backend_status[i] == CON_DOWN)
+					{
+						if (BACKEND_INFO(i).backend_status != CON_DOWN)
+						{
+
+							BACKEND_INFO(i).backend_status = CON_DOWN;
+							my_backend_status[i] = &(BACKEND_INFO(i).backend_status);
+							reload_maste_node_id = true;
+							ereport(LOG,
+									(errmsg("backend status from \"%s\" backend:%d is set to down status",backendStatus->nodeName, i)));
+						}
+					}
+					else if (backendStatus->backend_status[i] == CON_CONNECT_WAIT ||
+								backendStatus->backend_status[i] == CON_UP)
+					{
+						if (BACKEND_INFO(i).backend_status != CON_CONNECT_WAIT)
+						{
+							BACKEND_INFO(i).backend_status = CON_CONNECT_WAIT;
+							my_backend_status[i] = &(BACKEND_INFO(i).backend_status);
+							reload_maste_node_id = true;
+						}
+					}
+				}
+
+				if (reload_maste_node_id)
+				{
+					Req_info->master_node_id = get_next_master_node();
+				}
+			}
+			pfree(backendStatus);
+		}
+		else
+		{
+			ereport(WARNING,
+				(errmsg("failed to get the backend status from the master watchdog node"),
+					 errdetail("using the local backend node status")));
+		}
+	}
+}
